local addonName, addonTab = ... --[[	addonName = The name of this addon as in the TOC file and folder name. addonTab = The shared addon table between the Lua files of this addon.]]
local L = addonTab.L -- Localization
local DejaMarkFrame = addonTab.mainFrame -- The main frame local reference.
addonTab.version = GetAddOnMetadata(addonName, "Version")
addonTab.addoninfo = addonTab.version
addonTab.WoWClient = {
	Retail = (WOW_PROJECT_ID == WOW_PROJECT_MAINLINE),
	Wrath = (WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC),
	TBCC = (WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC),
	Vanilla = (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC),
}

-- ######## --
-- Defaults --
-- ######## --
if not (addonTab.defaults) then
	addonTab.defaults = {}
end

addonTab.defaults.DejaMark = {
    maskcolor = {0, 0.75, 1, 0.33}, -- Default color of the mask when checked.
	maskcolorchecked = false, -- Default state of the masked Overlay when checked.
}

-- ##################### --
-- SavedVariables Loader --
-- ##################### --
if not (addonTab.loader) then
	addonTab.loader = CreateFrame("Frame")
		addonTab.loader:RegisterEvent("ADDON_LOADED")
		addonTab.loader:SetScript("OnEvent", function(self, event, ...)
			if (event == "ADDON_LOADED" and ... == addonName) then -- only run the ADDON_LOADED event handler for this addon
				addonTab.loader[event](self, ...) -- call event handlers
			end
		end)

	function addonTab.loader:ADDON_LOADED(addonName)
		if not (_G[addonName.."DBPC"]) then -- first time the addon is used
			print(addonName..": First time use, creating saved variables")
			_G[addonName.."DBPC"] = {} -- allocate a table to store your saved variables (the table will be saved to disk on logout)
			_G[addonName.."DBPC"] = addonTab.defaults -- initialize the saved variables table with default values
		end
		for k,v in pairs(addonTab.defaults) do -- check for missing entries in the saved variables table
			if not (_G[addonName.."DBPC"][k]) then
				print(addonName..": Missing entry in saved variables, adding it")
				_G[addonName.."DBPC"][k] = v
			end
		end
		for k,v in pairs(_G[addonName.."DBPC"]) do -- check for obsolete entries in the saved variables table
			if not (addonTab.defaults[k]) then
				print(addonName..": Obsolete entry in saved variables, removing it", k, v)
				_G[addonName.."DBPC"][k] = nil
			end
		end
		addonTab.db = _G[addonName.."DBPC"] -- make a reference to the saved variables table
	end
end

-- ######## DejaMark Options Panel ######## --
	local DejaMarkOptionsPanel = CreateFrame("Frame", addonName.."OptionsPanel", UIParent)
	if (addonTab.WoWClient.Retail) then
		Settings.RegisterAddOnCategory( Settings.RegisterCanvasLayoutCategory(DejaMarkOptionsPanel, addonName) )
	else
		DejaMarkOptionsPanel.name = "DejaMark"
		InterfaceOptions_AddCategory(DejaMarkOptionsPanel)
	end

	DejaMarkOptionsPanel:RegisterEvent("PLAYER_LOGOUT")
	DejaMarkOptionsPanel:RegisterEvent("PLAYER_LEAVING_WORLD")

	DejaMarkOptionsPanel.title = DejaMarkOptionsPanel:CreateFontString("DejaMarkOptionsPanelTitle", "ARTWORK", "GameFontNormalLarge")
		DejaMarkOptionsPanel.title:SetPoint("TOPLEFT", DejaMarkOptionsPanel, "TOPLEFT", 16, -16)
		DejaMarkOptionsPanel.title:SetFont("Fonts\\MORPHEUS.TTF", 32, "THINOUTLINE")
		DejaMarkOptionsPanel.title:SetTextColor(0, 192, 255, 1)
		DejaMarkOptionsPanel.title:SetText(addonName)

	DejaMarkOptionsPanel.version = DejaMarkOptionsPanel:CreateFontString("DejaMarkOptionsPanelVersion", "ARTWORK", "GameFontNormal")
		DejaMarkOptionsPanel.version:SetPoint("BOTTOMRIGHT", DejaMarkOptionsPanel, "BOTTOMRIGHT", -16, 32)
		DejaMarkOptionsPanel.version:SetText(addonName..": "..addonTab.addoninfo)
		DejaMarkOptionsPanel.version:SetJustifyH("RIGHT")
		DejaMarkOptionsPanel.version:SetTextColor(0, 0.75, 1, 1)

	DejaMarkOptionsPanel.WoWClient = DejaMarkOptionsPanel:CreateFontString("DejaMarkOptionsPanelWoWClient", "ARTWORK", "GameFontNormal")
		DejaMarkOptionsPanel.WoWClient:SetPoint("BOTTOMRIGHT", DejaMarkOptionsPanel, "BOTTOMRIGHT", -16, 16)
		DejaMarkOptionsPanel.WoWClient:SetText("WoW Client: "..(addonTab.WoWClient.Retail and "Retail" or addonTab.WoWClient.Wrath and "Wrath" or addonTab.WoWClient.TBCC and "TBCC" or addonTab.WoWClient.Vanilla and "Vanilla" or "Unknown"))
		DejaMarkOptionsPanel.WoWClient:SetJustifyH("RIGHT")
		DejaMarkOptionsPanel.WoWClient:SetTextColor(0, 0.75, 1, 1)

	DejaMarkOptionsPanel:SetScript("OnEvent", function(self, event, ...)
		if (event == "PLAYER_LOGOUT" or event == "PLAYER_LEAVING_WORLD") then
			addonTab.db.DejaMark.maskcolorchecked = false
			_G[addonName.."DBPC"] = addonTab.db
		end
	end)

	DejaMarkOptionsPanel:SetScript("OnShow", function(self)
		addonTab.toggleframe:Disable()
		_G["DejaMarkFrame"]:SetFrameStrata("TOOLTIP")
		_G["DejaMarkFrame"]:SetPoint("CENTER", DejaMarkOptionsPanel, "CENTER", 5, -5)
		_G["DejaMarkFrame"]:Show()
		addonTab.db.DejaMark.maskcolorchecked = true
	end)

	DejaMarkOptionsPanel:SetScript("OnHide", function(self)
		addonTab.toggleframe:Enable()
		_G["DejaMarkFrame"]:SetFrameStrata("BACKGROUND")
		_G["DejaMarkFrame"]:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		_G["DejaMarkFrame"]:Hide()
		addonTab.db.DejaMark.maskcolorchecked = false
	end)

-- ######## DejaMark Options Panel: Defaulkt Reset Button ######## --
local DMReset = CreateFrame("Button", "DMResetButton", DejaMarkOptionsPanel, "UIPanelButtonTemplate")
	DMReset:ClearAllPoints()
	DMReset:SetPoint("BOTTOMLEFT", 5, 5)
	DMReset:SetScale(1.0)
	DMReset:SetWidth(175)
	DMReset:SetHeight(30)
	_G[DMReset:GetName() .. "Text"]:SetText("Reset to Defaults")
	DMReset.tooltipText = 'Reset |cff00c0ffDejaMark|r to its default settings.|n|cffff0000WARNING! This will reload the UI!|r' --Creates a tooltip on mouseover.

	DMReset:SetScript("OnClick", function(self, button, down)
		addonTab.db = addonTab.defaults
		_G[addonName.."DBPC"] = addonTab.db
		DejaMarkOptionsPanel:Hide()
		ReloadUI()
	end)

-- ######## DejaMark Options Panel: Color Picker Button ######## --
local DMColorPicker = CreateFrame("Button", "DejaMarkColorPickerButton", DejaMarkOptionsPanel, "UIPanelButtonTemplate")
	DMColorPicker:ClearAllPoints()
	DMColorPicker:SetPoint("CENTER", DejaMarkOptionsPanel, "CENTER", 0, -80)
	DMColorPicker:SetScale(1.0)
	DMColorPicker:SetWidth(175)
	DMColorPicker:SetHeight(30)
	_G[DMColorPicker:GetName() .. "Text"]:SetText("Deployed Flare Color")
	DMColorPicker.tooltipText = 'Select a custom indication color for World Marker Flares currently deployed in the field.' --Creates a tooltip on mouseover.

	DMColorPicker:SetScript("OnClick", function(self, button, down)
		local maskcolor = addonTab.db.DejaMark.maskcolor
		local r, g, b, a = unpack(maskcolor)
		addonTab.ShowColorPicker(_G["DejaMarkIconButton8"].Overlay, r, g, b, a) -- Any icon texture 1 - 8 will do, just have to get the current color of any of them since they are all the same.
	end)

function addonTab.myColorCallback()
    local a, r, g, b = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
    ColorPickerFrame.colourBox:SetColorTexture(r, g, b, a)
    addonTab.db.DejaMark.maskcolor = {r, g, b, a}
	DM_DesatMarks()
end

function addonTab.myColorCancel()
    local r, g, b, a = unpack(addonTab.previousValues)
    ColorPickerFrame.colourBox:SetColorTexture(r, g, b, a)
    addonTab.db.DejaMark.maskcolor = {r, g, b, a}
	DM_DesatMarks()
end

function addonTab.ShowColorPicker(self, r, g, b, a)
	addonTab.previousValues = {r, g, b, a}
	ColorPickerFrame.colourBox = self
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a;
	ColorPickerFrame:SetColorRGB(r,g,b);
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = addonTab.myColorCallback, addonTab.myColorCallback, addonTab.myColorCancel
	ColorPickerFrame:Show()
end

ColorPickerFrame:HookScript("OnShow", function(self)
	ColorPickerFrame:ClearAllPoints()
	ColorPickerFrame:SetPoint("TOP", DejaMarkOptionsPanel, "TOP", 0, -32)
end)
